#-----------------------------------------------------------------------------
# Zoom-to-extent
# July 4, 2003
# Suggested by maclean.
# PUBLIC DOMAIN, by Ockham

#-----------------------------------------------------------------------------

import math
import poser
scene = poser.Scene()
RadToDeg = 360.0 / math.pi
DegToRad = math.pi / 360.0

#-----------------------------------------------------------------------------
MaxX =  -10000.0
MinX =   10000.0
#MaxY = -10000.0
#MinY =  10000.0
MaxZ =  -10000.0
MinZ =   10000.0
# Run a long max-min check on X,Y,Z of all WorldVerts
# in all 'solid' objects, leaving aside the cameras, lights, and such.
# For speed, sparsify the search a bit; never hit more than
# 100 verts in each part.

for OneActor in scene.Actors():
    if OneActor.IsBodyPart() or OneActor.IsProp():
            if OneActor.IsLight(): continue
            if OneActor.IsCamera(): continue
            if OneActor.Name() == "UNIVERSE": continue
            elif OneActor.Name() == "GROUND": continue
            try:
                OneGeom = OneActor.Geometry()
                VertCount = OneGeom.NumVertices()
                Skipper = VertCount / 100
                if Skipper < 1: Skipper = 1
                for i in range(0,VertCount,Skipper):
                    OneWV = OneGeom.WorldVertex(i)
                    X = OneWV.X()
                    #Y = OneWV.Y()
                    Z = OneWV.Z()
                    if   X < MinX: MinX = X
                    elif X > MaxX: MaxX = X
                    #if   Y < MinY: MinY = Y
                    #elif Y > MaxY: MaxY = Y
                    if   Z < MinZ: MinZ = Z
                    elif Z > MaxZ: MaxZ = Z
            except:
                pass

CenterX = (MaxX + MinX) / 2.0
#CenterY = (MaxY + MinY) / 2.0
CenterZ = (MaxZ + MinZ) / 2.0
ExtentX = abs(MaxX - MinX)
#ExtentY = abs(MaxY - MinY)
ExtentZ = abs(MaxZ - MinZ)
# For now, use the 'circumscribed circle' by simply taking the max
# of X and Y extents.  There should be a more elegant way to account
# for the camera's Y orbit, but I don't see it just yet.
# Y orbit goes clockwise, 0 at front. So if you're looking FROM due left,
# Y orbit = +90.
Extent = max(ExtentX, ExtentZ)

# Now figure the camera's Dolly Z from this extent and the focal length.
# Empirically, when fl = 38mm, DollyZ = .01 * Extent.
# To start with, assume 38, then see how the f actually goes.  Formulas
# given for this don't seem to work here at first glance.
# The given formula seems to give DollyZ = f * Extent * some K

OneCam = scene.CurrentCamera()

FocalParm = OneCam.Parameter("Focal")
DollyXParm = OneCam.Parameter("DollyX")
DollyZParm = OneCam.Parameter("DollyZ")
OrbitYParm = OneCam.Parameter("yOrbit")
# Before we modify anything, set the camera's Scale values to 1.
# This isn't exactly nice, but otherwise we have too many variables
# wiggling around.
try:
    OneCam.Parameter("xScale").SetValue(1.0)
except:
    pass
try:
    OneCam.Parameter("yScale").SetValue(1.0)
except:
    pass
try:
    OneCam.Parameter("zScale").SetValue(1.0)
except:
    pass
try:
    OneCam.Parameter( "Scale").SetValue(1.0)
except:
    pass


F = FocalParm.Value()
NewDollyZ = Extent * 0.001 * F
DollyZParm.SetValue(NewDollyZ)

# Next, slide the camera over (Dolly X) so that it is centered on
# the actual center.  This may work, if DollyX is not "proportional".
# Use vectors.
# One vector is the camera's line from center of Poser.
# Angle = OrbitY, R =  Dolly Z.
CamVecAngle = DegToRad * OrbitYParm.Value()
CamVecR     = DegToRad * DollyZParm.Value()

# The other vector goes from the Scene Center to center of Poser.
# Note: avoiding X and Y terminology here, because we already have
# X's and Y's that are not the same thing!

CenVecAngle = math.atan2(CenterX, CenterZ)
# The X dim occupies the 'opposite' position in atan
CenVecR     = math.sqrt(CenterX*CenterX + CenterZ*CenterZ)
# Next, make the CenVec relative to the CamVec by subtraction:
CenVecAngle = CenVecAngle - CamVecAngle
# Then we want the component of this vector that is max when
# it's perp to the CamVec and min when it's along the CamVec,
# so we want the sin.
NewDollyX = math.sin(CenVecAngle) * CenVecR
DollyXParm.SetValue(NewDollyX)



#--------------------------------------------------------------------------
